@@ -13,7 +13,7 @@ from scipy.sparse.csr import spmatrix
 from scipy.stats import chi2
 from typing import Mapping, Sequence, Tuple, Iterable, Union
 from scipy.sparse import issparse
-from sklearn.metrics import adjusted_rand_score, normalized_mutual_info_score, silhouette_score
+from sklearn.metrics import adjusted_rand_score, normalized_mutual_info_score, silhouette_samples
 from sklearn.neighbors import NearestNeighbors
 from scETM.logging_utils import log_arguments
 import psutil
@@ -65,7 +65,8 @@ def evaluate(adata: ad.AnnData,
         return_fig: whether to return the Figure object. Useful for visualizing
             the plot.
         color_by: a list of adata.obs column keys to color the embeddings by.
-            Only used if is drawing.
+            If None, will look up adata.uns['color_by']. Only used if is
+            drawing.
         plot_fname: file name of the generated plot. Only used if is drawing.
         plot_ftype: file type of the generated plot. Only used if is drawing.
         plot_dir: directory to save the generated plot. If None, do not save
@@ -85,10 +86,10 @@ def evaluate(adata: ad.AnnData,
         key "fig".
     """
 
-    if not pd.api.types.is_categorical_dtype(adata.obs[cell_type_col]):
+    if cell_type_col and not pd.api.types.is_categorical_dtype(adata.obs[cell_type_col]):
         _logger.warning("scETM.evaluate assumes discrete cell types. Converting cell_type_col to categorical.")
         adata.obs[cell_type_col] = adata.obs[cell_type_col].astype(str).astype('category')
-    if not pd.api.types.is_categorical_dtype(adata.obs[batch_col]):
+    if batch_col and not pd.api.types.is_categorical_dtype(adata.obs[batch_col]):
         _logger.warning("scETM.evaluate assumes discrete batches. Converting batch_col to categorical.")
         adata.obs[batch_col] = adata.obs[batch_col].astype(str).astype('category')
 
@@ -101,8 +102,12 @@ def evaluate(adata: ad.AnnData,
     else:
         cluster_key = best_ari = best_nmi = None
 
-    asw = silhouette_score(adata.X if embedding_key == 'X' else adata.obsm[embedding_key], adata.obs[cell_type_col])
+    sw = silhouette_samples(adata.X if embedding_key == 'X' else adata.obsm[embedding_key], adata.obs[cell_type_col])
+    adata.obs['silhouette_width'] = sw
+    asw = np.mean(sw)
     _logger.info(f'{embedding_key}_ASW: {asw:7.4f}')
+    if batch_col and cell_type_col:
+        _logger.info(f'SW: {adata.obs.pivot_table(index=cell_type_col, columns=batch_col, values="silhouette_width", aggfunc="mean")}')
 
     # calculate batch correction metrics
     need_batch = batch_col and adata.obs[batch_col].nunique() > 1
@@ -130,10 +135,13 @@ def evaluate(adata: ad.AnnData,
     if return_fig or plot_dir is not None:
         if color_by is None:
             color_by = [batch_col, cell_type_col] if need_batch else [cell_type_col]
+        color_by = list(color_by)
+        if 'color_by' in adata.uns:
+            for col in adata.uns['color_by']:
+                if col not in color_by:
+                    color_by.insert(0, col)
         if cluster_key is not None:
             color_by = [cluster_key] + color_by
-        if 'clf_pred' in adata.obs:
-            color_by = ['clf_pred', 'clf_correct'] + color_by
         fig = draw_embeddings(adata=adata, color_by=color_by, min_dist=min_dist, spread=spread,
             ckpt_dir=plot_dir, fname=f'{plot_fname}.{plot_ftype}', return_fig=return_fig)
         if writer is not None:
